@@ -443,9 +443,12 @@ def _get_device_dataframe(
 
 
 def _add_settings_to_dataframe(sdk, device_dataframe):
-    macos_guids = device_dataframe.loc[
-        device_dataframe["osName"] == "mac", "guid"
-    ].values
+    macos_guids = [
+        {"guid": value}
+        for value in device_dataframe.loc[
+            device_dataframe["osName"] == "mac", "guid"
+        ].values
+    ]
 
     def handle_row(guid):
         try:
