@@ -3,8 +3,8 @@ import functools
 import click
 from pandas import DataFrame
 from pandas import json_normalize
-from py42.exceptions import Py42BadRequestError
 from py42.exceptions import Py42NotFoundError
+from py42.exceptions import Py42UserRiskProfileNotFound
 
 from code42cli.bulk import generate_template_cmd_factory
 from code42cli.bulk import run_bulk_process
@@ -221,6 +221,10 @@ _bulk_user_roles_headers = ["username", "role_name"]
 
 _bulk_user_alias_headers = ["username", "alias"]
 
+_bulk_user_risk_profile_headers = ["username", "start_date", "end_date", "notes"]
+
+_bulk_user_activation_headers = ["username"]
+
 
 @users.command(name="move")
 @username_option("The username of the user to move.", required=True)
@@ -252,13 +256,73 @@ def remove_alias(state, username, alias):
     _remove_cloud_alias(state.sdk, username, alias)
 
 
+@users.command()
+@click.argument("username")
+@click.argument(
+    "date", type=click.DateTime(formats=["%Y-%m-%d"]), required=False, metavar="DATE"
+)
+@click.option("--clear", is_flag=True, help="Clears the current `start_date` value.")
+@sdk_options()
+def update_start_date(state, username, date, clear):
+    """Sets the `start_date` on a User's risk profile (useful for users on the New Hire Watchlist).
+    Date format: %Y-%m-%d"""
+    if not date and not clear:
+        raise Code42CLIError("Must supply DATE argument if --clear is not used.")
+    if clear:
+        date = ""
+    user_id = _get_user(state.sdk, username)["userId"]
+    state.sdk.userriskprofile.update(user_id, start_date=date)
+
+
+@users.command()
+@click.argument("username")
+@click.argument("date", type=click.DateTime(formats=["%Y-%m-%d"]), required=False)
+@click.option("--clear", is_flag=True, help="Clears the current `end_date` value.")
+@sdk_options()
+def update_departure_date(state, username, date, clear):
+    """Sets the `end_date` on a User's risk profile (useful for users on the Departing Watchlist). Date format: %Y-%m-%d"""
+    if not date and not clear:
+        raise Code42CLIError("Must supply DATE argument if --clear is not used.")
+    if clear:
+        date = ""
+    user_id = _get_user(state.sdk, username)["userId"]
+    state.sdk.userriskprofile.update(user_id, end_date=date)
+
+
+@users.command()
+@click.argument("username")
+@click.argument("note", required=False)
+@click.option("--clear", is_flag=True, help="Clears the current `notes` value.")
+@click.option(
+    "--append",
+    is_flag=True,
+    help="Appends provided note to existing note text as a new line.",
+)
+@sdk_options()
+def update_risk_profile_notes(state, username, note, clear, append):
+    """Sets the `notes` value of a User's risk profile.
+
+    WARNING: Overwrites any existing note value."""
+    if not note and not clear:
+        raise Code42CLIError("Must supply NOTE argument if --clear is not used.")
+    user = _get_user(state.sdk, username)
+    user_id = user["userId"]
+    if append and user["notes"]:
+        note = user["notes"] + f"\n\n{note}"
+    if clear:
+        note = ""
+    state.sdk.userriskprofile.update(user_id, notes=note)
+
+
 @users.command()
 @click.argument("username")
 @sdk_options()
 def list_aliases(state, username):
-    """List the cloud aliases for a given user.  Each user has a default cloud alias of their Code42 username with up to one additional alias."""
+    """List the cloud aliases for a given user.
+
+    Each user has a default cloud alias of their Code42 username with up to one additional alias."""
     user = _get_user(state.sdk, username)
-    aliases = user["cloudUsernames"]
+    aliases = user["cloudAliases"]
     if aliases:
         click.echo(aliases)
     else:
@@ -336,6 +400,7 @@ users_generate_template = generate_template_cmd_factory(
         "move": _bulk_user_move_headers,
         "add-alias": _bulk_user_alias_headers,
         "remove-alias": _bulk_user_alias_headers,
+        "update-risk-profile": _bulk_user_risk_profile_headers,
     },
     help_message="Generate the CSV template needed for bulk user commands.",
 )
@@ -422,9 +487,6 @@ def bulk_move(state, csv_rows, format):
     formatter.echo_formatted_list(result_rows)
 
 
-_bulk_user_activation_headers = ["username"]
-
-
 @bulk.command(
     name="deactivate",
     help=f"Deactivate a list of users from the provided CSV in format: {','.join(_bulk_user_activation_headers)}",
@@ -663,6 +725,50 @@ def bulk_remove_alias(state, csv_rows, format):
     formatter.echo_formatted_list(result_rows)
 
 
+@bulk.command(
+    name="update-risk-profile",
+    help=f"Update user risk profile data from the provided CSV in format: {','.join(_bulk_user_risk_profile_headers)}"
+    "\n\nTo clear a value, set column item to the string: 'null'.",
+)
+@format_option
+@read_csv_arg(headers=_bulk_user_risk_profile_headers)
+@click.option(
+    "--append-notes",
+    is_flag=True,
+    help="Append provided note value to already existing note on a new line. Defaults to overwrite.",
+)
+@sdk_options()
+def bulk_update_risk_profile(state, csv_rows, format, append_notes):
+    """Bulk update User Risk Profile data."""
+    sdk = state.sdk
+
+    success_header = "updated_user"
+    formatter = OutputFormatter(
+        format, {key: key for key in [*csv_rows[0].keys(), success_header]}
+    )
+    stats = create_worker_stats(len(csv_rows))
+
+    def handle_row(**row):
+        try:
+            updated_user = _update_userriskprofile(
+                sdk, append_notes=append_notes, **row
+            )
+            row[success_header] = updated_user
+        except Exception as err:
+            row[success_header] = f"Error: {err}"
+            stats.increment_total_errors()
+        return row
+
+    result_rows = run_bulk_process(
+        handle_row,
+        csv_rows,
+        progress_label="Updating user risk profile data:",
+        stats=stats,
+        raise_global_error=False,
+    )
+    formatter.echo_formatted_list(result_rows)
+
+
 def _add_user_role(sdk, username, role_name):
     user_id = _get_legacy_user_id(sdk, username)
     _get_role_id(sdk, role_name)  # function provides role name validation
@@ -793,18 +899,45 @@ def _reactivate_user(sdk, username):
 
 
 def _get_user(sdk, username):
-    # use when retrieving the user information from the detectionlists module
+    # use when retrieving the user risk profile information
     try:
-        return sdk.detectionlists.get_user(username).data
-    except Py42BadRequestError:
+        return sdk.userriskprofile.get_by_username(username)
+    except Py42UserRiskProfileNotFound:
         raise UserDoesNotExistError(username)
 
 
 def _add_cloud_alias(sdk, username, alias):
     user = _get_user(sdk, username)
-    sdk.detectionlists.add_user_cloud_alias(user["userId"], alias)
+    sdk.userriskprofile.add_cloud_aliases(user["userId"], alias)
 
 
 def _remove_cloud_alias(sdk, username, alias):
     user = _get_user(sdk, username)
-    sdk.detectionlists.remove_user_cloud_alias(user["userId"], alias)
+    sdk.userriskprofile.delete_cloud_aliases(user["userId"], alias)
+
+
+def _update_userriskprofile(
+    sdk, append_notes=False, username=None, start_date=None, end_date=None, notes=None
+):
+    user = _get_user(sdk, username)
+    user_id = user["userId"]
+    if append_notes and notes != "null":
+        notes = user["notes"] + f"\n\n{notes}"
+
+    # py42 interprets empty string as "clear this value" for kwarg values. Since empty CSV columns
+    # get parsed as "" we want to have user provide explicit 'null' string to indicate desire to
+    # clear instead of just not update value
+    start_date = (
+        None if start_date == "" else ("" if start_date == "null" else start_date)
+    )
+    end_date = None if end_date == "" else ("" if end_date == "null" else end_date)
+    notes = None if notes == "" else ("" if notes == "null" else notes)
+
+    updated_user = sdk.userriskprofile.update(
+        user_id, start_date=start_date, end_date=end_date, notes=notes
+    )
+    return {
+        k: v
+        for k, v in updated_user.data.items()
+        if k in ["username", "userId", "startDate", "endDate", "notes"]
+    }
