@@ -7,10 +7,10 @@ from code42cli.cmds.search.options import BeginOption
 from code42cli.date_helper import convert_datetime_to_timestamp
 from code42cli.date_helper import round_datetime_to_day_end
 from code42cli.date_helper import round_datetime_to_day_start
+from code42cli.enums import OutputFormat
+from code42cli.enums import SendToFileEventsOutputFormat
 from code42cli.errors import Code42CLIError
 from code42cli.logger.enums import ServerProtocol
-from code42cli.output_formats import OutputFormat
-from code42cli.output_formats import SendToFileEventsOutputFormat
 from code42cli.profile import get_profile
 from code42cli.sdk_client import create_sdk
 
@@ -165,7 +165,7 @@ send_to_format_options = click.option(
 def begin_option(term, **kwargs):
     defaults = dict(
         type=MagicDate(rounding_func=round_datetime_to_day_start),
-        help=f"The beginning of the date range in which to look for {term}. {MagicDate.HELP_TEXT}",
+        help=f"The beginning of the date range in which to look for {term}. {MagicDate.HELP_TEXT} [required unless --use-checkpoint option used]",
         cls=BeginOption,
         callback=lambda ctx, param, arg: convert_datetime_to_timestamp(arg),
     )
@@ -186,7 +186,11 @@ def end_option(term, **kwargs):
 
 
 def checkpoint_option(term, **kwargs):
-    defaults = dict(help=f"Only get {term} that were not previously retrieved.")
+    defaults = dict(
+        help=f"Use a checkpoint with the given name to only get {term} that were not previously retrieved."
+        f"If a checkpoint for {term} with the given name doesn't exist, it will be created on the first run."
+        "Subsequent CLI runs with this flag and the same name will use the stored checkpoint to modify the search query and then update the stored checkpoint"
+    )
     defaults.update(kwargs)
     return click.option("-c", "--use-checkpoint", **defaults)
 
@@ -206,3 +210,11 @@ def set_end_default_dict(term):
         "the same as `--begin`.",
         callback=lambda ctx, param, arg: convert_datetime_to_timestamp(arg),
     )
+
+
+column_option = click.option(
+    "--columns",
+    default=None,
+    callback=lambda ctx, param, value: value.split(",") if value is not None else None,
+    help="Filter output to include only specified columns. Accepts comma-separated list of column names (case-insensitive).",
+)
